from datetime import timezone

from obspy import UTCDateTime
from pydantic import field_serializer, BaseModel
from typing import List, Optional, Dict, Any

from .MetadataCategory import MetadataCategory
from ..pydantic_utcdatetime import CustomUTCDateTimeType


class MetadataQuery(BaseModel):
    id: Optional[int] = None
    category: Optional[MetadataCategory] = None
    starttime: Optional[CustomUTCDateTimeType] = None
    endtime: Optional[CustomUTCDateTimeType] = None
    created_after: Optional[CustomUTCDateTimeType] = None
    created_before: Optional[CustomUTCDateTimeType] = None
    network: Optional[str] = None
    station: Optional[str] = None
    channel: Optional[str] = None
    location: Optional[str] = None
    data_valid: Optional[bool] = None
    metadata: Optional[Dict[str, Any]] = None
    status: Optional[List[str]] = None

    # instructions for model_dump() to serialize pydantic CustomUTCDateTimeType into aware datetime.datetime type
    # sqlalchemy is expecting aware datetime.datetime, not the string model_dump() creates by default
    @field_serializer("created_after")
    def serialize_created_after(self, created_after: UTCDateTime):
        if created_after is not None:
            created_after = created_after.datetime.replace(tzinfo=timezone.utc)
        return created_after

    @field_serializer("created_before")
    def serialize_created_before(self, created_before: UTCDateTime):
        if created_before is not None:
            created_before = created_before.datetime.replace(tzinfo=timezone.utc)
        return created_before

    @field_serializer("starttime")
    def serialize_starttime(self, starttime: UTCDateTime):
        if starttime is not None:
            starttime = starttime.datetime.replace(tzinfo=timezone.utc)
        return starttime

    @field_serializer("endtime")
    def serialize_endtime(self, endtime: UTCDateTime):
        if endtime is not None:
            endtime = endtime.datetime.replace(tzinfo=timezone.utc)
        return endtime
